import os
import glob
import optparse
import re
import sys
import time

import emdash.config
import emdash.proxy
import emdash.handlers


def main():
	parser = emdash.config.DBRPCOptions()
	usage = """%prog download [options] <name>"""
	parser.set_usage(usage)
	parser.add_option("--recurse", type="int",help="Recursion level", default=3)
	parser.add_option("--overwrite", "-o", action="store_true", help="Overwrite existing files (default is to skip)", default=False)
	parser.add_option("--rename", "-r", action="store_true", help="If a file already exists, save with format 'rename.originalfilename'", default=False)
	parser.add_option("--sidecar", "-s", action="store_true", help="Include sidecar file with EMEN2 metadata in JSON format")
	parser.add_option("--gzip", action="store_true", dest="compress", help="Decompress gzip'd files. Requires gzip in path (default)", default=True)
	parser.add_option("--nogzip", action="store_false", dest="compress", help="Do not decompress gzip'd files.")
	# parser.add_option("--bdorename", "-b", action="store_true", help="Rename to 'bdo.YYYYMMDDXXXXX.extension'")
	(options, args) = parser.parse_args()
	db = parser.opendb(request=True)

	opts = {}
	for k,v in options.__dict__.items():
		opts[k] = v

	for name in args:
		rec = db.getrecord(int(name))
		handler = emdash.handlers.gethandler(rectype=rec.get('rectype'))
		dbt = handler(db=db, name=name, options=opts, pos=(0,1))
		dbt.download()



if __name__ == "__main__":
	main()
